==> openscad_2013.01+dfsg.orig.tar.gz: openscad-2013.01+dfsg.orig/src/csgops.cc <==
/*
 *  OpenSCAD (www.openscad.org)
 *  Copyright (C) 2009-2011 Clifford Wolf <clifford@clifford.at> and
 *                          Marius Kintel <marius@kintel.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  As a special exception, you have permission to link this program
 *  with the CGAL library and distribute executables, as long as you
 *  follow the requirements of the GNU GPL in regard to all of the
 *  software in the executable aside from CGAL.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "csgnode.h"

#include "module.h"
#include "csgterm.h"
#include "builtin.h"
#include <sstream>
#include <assert.h>

class CsgModule : public AbstractModule
{
public:
	csg_type_e type;
	CsgModule(csg_type_e type) : type(type) { }
	virtual AbstractNode *evaluate(const Context *ctx, const ModuleInstantiation *inst) const;
};

AbstractNode *CsgModule::evaluate(const Context*, const ModuleInstantiation *inst) const
{
	CsgNode *node = new CsgNode(inst, type);
	std::vector<AbstractNode *> evaluatednodes = inst->evaluateChildren();
	node->children.insert(node->children.end(), evaluatednodes.begin(), evaluatednodes.end());
	return node;
}

std::string CsgNode::toString() const
{
	return this->name() + "()";
}

std::string CsgNode::name() const
{
	switch (this->type) {
	case CSG_TYPE_UNION:
		return "union";
		break;
	case CSG_TYPE_DIFFERENCE:
		return "difference";
		break;
	case CSG_TYPE_INTERSECTION:
		return "intersection";
		break;
	default:
		assert(false);
	}
}

void register_builtin_csgops()
{
	Builtins::init("union", new CsgModule(CSG_TYPE_UNION));
	Builtins::init("difference", new CsgModule(CSG_TYPE_DIFFERENCE));
	Builtins::init("intersection", new CsgModule(CSG_TYPE_INTERSECTION));
}

==> openscad_2015.03-1+dfsg.orig.tar.gz: openscad-2015.03-1+dfsg/src/csgops.cc <==
/*
 *  OpenSCAD (www.openscad.org)
 *  Copyright (C) 2009-2011 Clifford Wolf <clifford@clifford.at> and
 *                          Marius Kintel <marius@kintel.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  As a special exception, you have permission to link this program
 *  with the CGAL library and distribute executables, as long as you
 *  follow the requirements of the GNU GPL in regard to all of the
 *  software in the executable aside from CGAL.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "csgnode.h"

#include "evalcontext.h"
#include "module.h"
#include "csgterm.h"
#include "builtin.h"
#include <sstream>
#include <assert.h>

class CsgModule : public AbstractModule
{
public:
	OpenSCADOperator type;
	CsgModule(OpenSCADOperator type) : type(type) { }
	virtual AbstractNode *instantiate(const Context *ctx, const ModuleInstantiation *inst, EvalContext *evalctx) const;
};

AbstractNode *CsgModule::instantiate(const Context*, const ModuleInstantiation *inst, EvalContext *evalctx) const
{
	inst->scope.apply(*evalctx);
	CsgNode *node = new CsgNode(inst, type);
	std::vector<AbstractNode *> instantiatednodes = inst->instantiateChildren(evalctx);
	node->children.insert(node->children.end(), instantiatednodes.begin(), instantiatednodes.end());
	return node;
}

std::string CsgNode::toString() const
{
	return this->name() + "()";
}

std::string CsgNode::name() const
{
	switch (this->type) {
	case OPENSCAD_UNION:
		return "union";
		break;
	case OPENSCAD_DIFFERENCE:
		return "difference";
		break;
	case OPENSCAD_INTERSECTION:
		return "intersection";
		break;
	default:
		assert(false);
	}
	return "internal_error";
}

void register_builtin_csgops()
{
	Builtins::init("union", new CsgModule(OPENSCAD_UNION));
	Builtins::init("difference", new CsgModule(OPENSCAD_DIFFERENCE));
	Builtins::init("intersection", new CsgModule(OPENSCAD_INTERSECTION));
}
